#!/usr/bin/env python3
# +------------------------------------------------------------+
# |                                                            |
# |             | |             | |            | |             |
# |          ___| |__   ___  ___| | ___ __ ___ | | __          |
# |         / __| '_ \ / _ \/ __| |/ / '_ ` _ \| |/ /          |
# |        | (__| | | |  __/ (__|   <| | | | | |   <           |
# |         \___|_| |_|\___|\___|_|\_\_| |_| |_|_|\_\          |
# |                                   custom code by SVA       |
# |                                                            |
# +------------------------------------------------------------+
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright (C) 2022, 2023 SVA System Vertrieb Alexander GmbH
#                            by michael.hoess@sva.de


from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    any_of,
    exists,
    contains,
    check_levels,
    get_value_store,    
    register,
    render,
    Metric,
    OIDEnd,
    Result,
    Service,
    SNMPTree,
    State,            
)




#
# Section plugin
#

def parse_akcp_sensors(string_table):
    section={}
    for snmpsect in string_table:
        for ln in snmpsect:
            name = ln[0] 
            
            sens_type = ln[1]
            unit = ln[2]
            values =  ln[3:]

            typ = "Unknown"
            if name.find("Temp") >= 0 or sens_type == 3:
                typ = "Temperature"            

            elif name.find("Humi") >= 0 or name.find("Feucht") >= 0  or sens_type in [ 32, 46 ]:
                typ = "Humidity"            

            # Convention "Sensort Port 1"
            devid = name.split(" ")[-1]
            # Convention "Sensor_1"
            if name.find("_") >= 0:
                devid = name.split("_")[-1]

            cvalues = []
            for v in values:
                nv = None
                try:
                    nv = float(v)
                    if typ == "Temperature":
                        nv = nv / 10
                except:
                    pass
                cvalues.append(nv)
            

            id = f"AKCP {typ} #{devid}"

            section[id] = {
                "name": name,
                "typ": typ,
                "unit": unit,
                "val": cvalues[0],
                "levels_low": (cvalues[2], cvalues[1]),
                "levels_high": (cvalues[3], cvalues[4])
            }
    
    return section


register.snmp_section(
    name="akcp_sensors",
    parse_function=parse_akcp_sensors,
    fetch=[
        SNMPTree(
            base=".1.3.6.1.4.1.3854.3.5.2.1", # temp sensors
            oids=[
                "2",  # Name
                "3",  # Type
                "5",  # Unit
                "20", # Value * 10
                "9",  # Low crit
                "10", # Low warn
                "11", # High warn
                "12", # High ceir
        ]),
        SNMPTree(
            base=".1.3.6.1.4.1.3854.3.5.3.1", # humidity sensors
            oids=[
                "2",  # Name
                "3",  # Type
                "5",  # Unit
                "20", # Value * 1
                "9",  # Low crit
                "10", # Low warn
                "11", # High warn
                "12", # High warn
            ]),
        ],
    detect=any_of(
        exists(".1.3.6.1.4.1.3854"), # akcp mib
        contains(".1.3.6.1.4.1.3854.3.2.1.8.0", "SP2"), # Part of device/firmware name,  may contain SP2 or SP2+
    ),
)




#
# Common discovery
#

def discover_akcp_sensors(section, params, filter):    
    res = []
    for k, v in section.items():
        if k.find(filter) >= 0:
            res.append(Service(item=k))

    return res





#
# Temperature plugin
#

def check_akcp_sensors_temperature(item, params, section):

    data = section.get(item)    
    if data:
        ll_hi = params["levels_upper"] if "levels_upper" in params else data["levels_high"]
        ll_lo = params["levels_lower"] if "levels_lower" in params else data["levels_low"]

        yield from check_levels(
            value=data["val"],
            levels_lower=ll_lo,
            levels_upper=ll_hi,
            render_func=lambda x: f"{float(x)} °{data['unit']}",
            metric_name="temp"
        )

register.check_plugin(
    name="akcp_sensors_temperature",
    sections=['akcp_sensors'],
    service_name="%s",
    discovery_function=lambda section: (yield from discover_akcp_sensors(
        section,
        {},
        'Temp'
    )),
    check_function=check_akcp_sensors_temperature,
    check_ruleset_name='check_akcp_sensors_temperature',
    check_default_parameters={}, 
)





#
# Humidity plugin
#

def check_akcp_sensors_humidity(item, params, section):
    
    data = section.get(item)
    if data:
        ll_hi = params["levels_upper"] if "levels_upper" in params else data["levels_high"]
        ll_lo = params["levels_lower"] if "levels_lower" in params else data["levels_low"]
        
        yield from check_levels(
            value=data["val"],
            levels_lower=ll_lo,
            levels_upper=ll_hi,
            render_func=lambda x: f"{int(x)}% rH",
            metric_name="humidity"
        )


register.check_plugin(
    name="akcp_sensors_humidity",
    sections=['akcp_sensors'],
    service_name="%s",
    discovery_function=lambda section: (yield from discover_akcp_sensors(
        section,
        {},
        'Humi'
    )),
    check_function=check_akcp_sensors_humidity,
    check_ruleset_name='check_akcp_sensors_humidity',
    check_default_parameters={}, 
)
